/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYAPPLICATIONCONTEXT_H_
#define FANCYAPPLICATIONCONTEXT_H_

#include <fancybase.h>
#include <fancyobject.h>

namespace Fancy {

class Map;

/**
 * An application context holding stateless beans e.g. for accessing
 * remote resources like databases or remote servers.
 */
class FANCYBASE_API ApplicationContext: public virtual Fancy::Object {

    ObjPtr<Map> beans;

public:
    typedef ObjPtr<ApplicationContext> ptr_t;

    /**
     * @return A new standalone application context.
     */
    static ptr_t newInstance();

    /**
     * @return The static default application context.
     *         If the static has not been initialized before,
     *         it is lazily created by this call.
     */
    static ptr_t getStaticInstance();

    /**
     * Release the static instance. calling this method is
     * encouraged at the end of your main program in order to
     * facilitate error logging during application context shutdown.
     */
    static void releaseStaticInstance();

    /**
     * @param name The name of the bean to retrieve.
     * @return A bean instance. If no bean is registered under
     *         the given @c name, an exception is thrown.
     */
    ObjPtr<Object> getBean(const char *name) const;

    /**
     * Register a bean under the given name.
     * @param name The name of the bean to register.
     * @param bean The bean to set.
     */
    void setBean(const char *name, const ObjPtr<Object> &bean);

    /**
     * @param name The name of the bean to retrieve.
     * @return A bean instance of type @c T. If no bean is registered under
     *         the given @c name or the registered bean is not of type @c t,
     *         an exception is thrown.
     */
    template<class T>
    ObjPtr<T> get(const char *name) const
    { return this->getBean(name).check_cast<T>(); }

    /**
     * @param name The name of the bean to retrieve from the static instance.
     * @return A bean instance. If no bean is registered under
     *         the given @c name, an exception is thrown.
     */
    static ObjPtr<Object> getBean_s(const char *name);

    /**
     * Register a bean in the static instance under the given name.
     * @param name The name of the bean to register.
     * @param bean The bean to set.
     */
    static void setBean_s(const char *name, const ObjPtr<Object> &bean);

    /**
     * @param name The name of the bean to retrieve from the static instance.
     * @return A bean instance of type @c T. If no bean is registered under
     *         the given @c name or the registered bean is not of type @c t,
     *         an exception is thrown.
     */
    template<class T> static
    ObjPtr<T> get_s(const char *name)
    { return ApplicationContext::getBean_s(name).check_cast<T>(); }

    /**
     * @param name The name of the bean of interest.
     * @return Whether a bean is registered under the given @c name.
     */
    bool hasBean(const char *name) const;

    /**
     * @param name The name of the bean of interest.
     * @return Whether a bean is registered under the given @c name.
     */
    static bool hasBean_s(const char *name);

protected:
    ApplicationContext();
    virtual ~ApplicationContext();
};

}

#endif /* FANCYAPPLICATIONCONTEXT_H_ */
