/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCY_BINARY_HELPER_H_
#define FANCY_BINARY_HELPER_H_

#include <fancyobject.h>

namespace Fancy {

//class List;
template<class T> class ArrayObject;


namespace BinaryHelper {

// byte array from an to hex strings


/**
 * Parses a hex char into the lower 4 bits (0x0F) of one uint8_t.
 */
uint8_t FANCYBASE_API parseHexChar(const unsigned char hex);

/**
 * Parses the hex String into an array of uint8_t.
 * hex String must NOT CONTAIN WHITE SPACE like ' ', '\t', '\n', '\r' - use StringHelper::removeWhiteSpace() if unsure
 */
Fancy::ObjPtr<Fancy::ArrayObject<uint8_t> > FANCYBASE_API hexStringToUInt8Array(const char* hex);

/**
 * Parses the hex String into an array of int8_t.
 * hex String must NOT CONTAIN WHITE SPACE like ' ', '\t', '\n', '\r' - use StringHelper::removeWhiteSpace() if unsure.
 */
Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > FANCYBASE_API hexStringToInt8Array(const char* hex);

/**
 * Parses a bit char into the lowest bit (0x01) of one uint8_t.
 */
uint8_t FANCYBASE_API parseBitChar(const unsigned char b);

/**
 * Parses the bit String into an array of uint8_t.
 * hex String must NOT CONTAIN WHITE SPACE like ' ', '\t', '\n', '\r' - use StringHelper::removeWhiteSpace() if unsure.
 */
Fancy::ObjPtr<Fancy::ArrayObject<uint8_t> > FANCYBASE_API bitStringToUInt8Array(const char* bitstring);

/**
 * Parses the bit String into an array of iint8_t.
 * hex String must NOT CONTAIN WHITE SPACE like ' ', '\t', '\n', '\r' - use StringHelper::removeWhiteSpace() if unsure.
 */
Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > FANCYBASE_API bitStringToInt8Array(const char* bitstring);

/**
 * Return the bit length of the given byte array, when being interpreted
 * as a big endian numbers as in cryptography.
 *
 * If @c ba is null and len is not zero, an exception is thrown.
 */
size_t FANCYBASE_API getBitLength(const void *ba, size_t len);

/**
 * Return the bit length of the given byte array, when being interpreted
 * as a big endian numbers as in cryptography.
 *
 * If @c ba is null, an exception is thrown.
 */
size_t FANCYBASE_API getBitLength(const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > &ba);

/**
 * Return the bit length of the given byte array, when being interpreted
 * as a big endian numbers as in cryptography.
 *
 * If @c ba is null, an exception is thrown.
 */
size_t FANCYBASE_API getBitLength(const Fancy::ObjPtr<Fancy::ArrayObject<uint8_t> > &ba);

/**
 * Formats the first len bytes pointed to by data as hex string.
 * If sep != '\0', it is inserted between bytes.
 * If maxLen > 11, the string is shortenend accordingly using "..." in the middle.
 * If data is NULL or len == 0, an empty string is returned.
 */
std::string FANCYBASE_API formatAsHexString(uint8_t* data, size_t len, char sep='\0', size_t maxLen=0);

/**
 * Formats the byte array as hex string.
 * If sep != '\0', it is inserted between bytes.
 * If maxLen > 11, the string is shortenend accordingly using "..." in the middle.
 * If byteArray is NULL or empty, an empty string is returned.
 */
std::string FANCYBASE_API formatAsHexString(const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > bytes, char sep='\0', size_t maxLen=0);

/**
 * Formats the byte array as hex string.
 * If sep != '\0', it is inserted between bytes.
 * If maxLen > 11, the string is shortenend accordingly using "...." in the middle.
 * If byteArray is NULL or empty, an empty string is returned.
 */
std::string FANCYBASE_API formatAsHexString(const Fancy::ObjPtr<Fancy::ArrayObject<uint8_t> > bytes, char sep='\0', size_t maxLen=0);

/**
 * Similar to formatAsHexString(), but returns "- (NULL)" or "- (empty)" or "x bytes: "+formatAsHexString()
 */
std::string FANCYBASE_API describeAsHexString(const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > bytes, char sep='\0', size_t maxLen=0);

/**
 * Similar to formatAsHexString(), but returns "- (NULL)" or "- (empty)" or "x bytes: "+formatAsHexString()
 */
std::string FANCYBASE_API describeAsHexString(const Fancy::ObjPtr<Fancy::ArrayObject<uint8_t> > bytes, char sep='\0', size_t maxLen=0);

/**
 * Formats the first length bytes data points to as hex dump.
 */
std::string FANCYBASE_API formatAsHexDump(uint8_t* data, int length, const char* indentation = 0);

/**
 * Formats the byte array as hex dump.
 * If length > 0, only that much bytes are dumped.
 */
std::string FANCYBASE_API formatAsHexDump(const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > bytes, int length = -1, const char* indentation = 0);

/**
 * Formats the byte array as hex dump.
 * If length > 0, only that much bytes are dumped.
 */
std::string FANCYBASE_API formatAsHexDump(const Fancy::ObjPtr<Fancy::ArrayObject<uint8_t> > bytes, int length = -1, const char* indentation = 0);


/**
 * Formats the first length bytes of data as bit string.
 */
std::string FANCYBASE_API formatAsBitString(uint8_t* data, int length);

/**
 * Formats the byte array as bit string.
 */
std::string FANCYBASE_API formatAsBitString(const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > bytes);

/**
 * Formats the byte array as bit string.
 */
std::string FANCYBASE_API formatAsBitString(const Fancy::ObjPtr<Fancy::ArrayObject<uint8_t> > bytes);


// ITA2

/**
 * Interprets the lowest 5 bit of b (0x1F) and returns the according character, following the ITA2 encoding.
 * See http://en.wikipedia.org/wiki/Baudot_code#ITA2, column "msb on left"
 */
char FANCYBASE_API decodeITA2Letter(uint8_t b);

/**
 * Interprets the lowest 5 bit of b (0x1F) and returns the according character, following the ITA2 encoding.
 * See http://en.wikipedia.org/wiki/Baudot_code#ITA2, column "msb on left"
 */
char FANCYBASE_API decodeITA2Figure(uint8_t b);

/**
 * ITA2-Encodes c into the lowest 5 bit of the returned uint8_t
 * See http://en.wikipedia.org/wiki/Baudot_code#ITA2, column "msb on left"
 */
uint8_t FANCYBASE_API encodeLetter2ITA(char c);

/**
 * ITA2-Encodes c into the lowest 5 bit of the returned uint8_t
 * See http://en.wikipedia.org/wiki/Baudot_code#ITA2, column "msb on left"
 */
uint8_t FANCYBASE_API encodeFigure2ITA(char c);

/**
 * Returns a uint8_t where the lowest 5 bit are in reverse order compared to b.
 */
uint8_t FANCYBASE_API reverseITA2BitOrder(uint8_t b);


// numbers in arrays

/**
 * Interprets the 8 bytes starting at where dataPtr points to as uint64
 * The caller is responsible to make sure it is ok to read those bytes.
 */
uint64_t FANCYBASE_API decodeULong(void * dataPtr);

/**
 * Interprets the 4 bytes starting at where dataPtr points to as uint32
 * The caller is responsible to make sure it is ok to read those bytes.
 */
uint32_t FANCYBASE_API decodeUInt(void * dataPtr);

/**
 * Interprets the 2 bytes starting at where dataPtr points to as uint16
 * The caller is responsible to make sure it is ok to read those bytes.
 */
uint16_t FANCYBASE_API decodeUShort(void * dataPtr);

/**
 * Encodes the value into the 8 bytes starting at where dataPtr points to.
 * The caller is responsible to make sure it is ok to write to those bytes.
 */
void FANCYBASE_API encodeULong(uint64_t value, void * dataPtr);

/**
 * Encodes the value into the 4 bytes starting at where dataPtr points to.
 * The caller is responsible to make sure it is ok to write to those bytes.
 */
void FANCYBASE_API encodeUInt(uint32_t value, void * dataPtr);

/**
 * Encodes the value into the 2 bytes starting at where dataPtr points to.
 * The caller is responsible to make sure it is ok to write to those bytes.
 */
void FANCYBASE_API encodeUShort(uint16_t value, void * dataPtr);


/**
 * Interprets the numBytes bytes starting at where dataPtr points to as BCD number
 * The caller is responsible to make sure it is ok to read those bytes.
 */
uint64_t FANCYBASE_API decodeBCD(void * dataPtr, size_t numBytes);

/**
 * Encodes the value into the numBytes bytes starting at where dataPtr points to.
 * If the number is too large to encode, an exception is thrown.
 * If the number requires less than numBytes bytes, it is right-justified, i.e. unneded early bytes are set to 0.
 * The caller is responsible to make sure it is ok to write to those bytes.
 */
void FANCYBASE_API encodeBCD(uint64_t value, void * dataPtr, size_t numBytes);


} // namespace BinaryHelper

} // namespace Fancy


#endif /* FANCY_BINARY_HELPER_H_ */
