/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYCONNECTIONPOOL_H_
#define FANCYCONNECTIONPOOL_H_

#define FANCY_CONNECTIONPOOL_BEAN "Fancy::ConnectionPool"

#include <fancyobject.h>

namespace Fancy {

class PoolableConnection;
class PoolableConnectionFactory;

class FANCYBASE_API ConnectionPool: public virtual Fancy::Object {
public:

    typedef ObjPtr<ConnectionPool> ptr_t;

    /**
     * Generate a new connection pool instance. The returned instance must
     * be activated by calling @c start() afterwards.
     *
     * @param minSize The minimal size of the connection pool.
     * @param maxSize The maximal size of the connection pool.
     * @param idleTimeout The time in milliseconds to wait before disposing
     *                    an idle connection.
     * @param waitTimeout The time in milliseconds to wait for a connection to become
     *                    available, when all connection are currently busy.
     *                    If no connection is available after @c waitTimeout
     *                    seconds, a time out exception is thrown.
     * @param factory The factory used to instantiate new connections.
     * @return A new @c ConnectionPool instance.
     */
    static ptr_t newInstance(size_t minSize, size_t maxSize,
                             unsigned idleTimeout, unsigned waitTimeout,
                             const ObjPtr<PoolableConnectionFactory> &factory);

    /**
     * Take a connection from the pool or wait for a connection to become available,
     * if all connections are busy.
     *
     * @return A connected connection, which has to be put nack lateron using
     *         @c putBackConnection.
     */
    virtual Fancy::ObjPtr<PoolableConnection> takeConnection() = 0;

    /**
     * Put back a connection after usage. If a connection is not put back, the
     * client pool will not work correctly.
     *
     * @param connection A connection previously fetched by @c takeConnection().
     */
    virtual void putBackConnection(const Fancy::ObjPtr<PoolableConnection> &connection) = 0;

    /**
     * Starts this connection pool and starts the garbage collection.
     */
    virtual void start() = 0;

    /**
     * Stop this connection pool and the garbage collector.
     * @c stop() is usually called inside the destructor, but
     * the user is encouraged to call stop() before the destructor.
     */
    virtual void stop() = 0;

protected:
    ConnectionPool() {}
    virtual ~ConnectionPool();
};

}

#endif /* FANCYCONNECTIONPOOL_H_ */
