/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2023 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCY_CYRYPT_HELPER_H_
#define FANCY_CYRYPT_HELPER_H_

#ifndef WIN32


#include <fancybase.h>

#include <string>


namespace Fancy {

namespace CryptHelper {


// see man 5 crypt

// 3x not supported:
//#define FANCY_CRYPTHELPER_PREFIX_YESCRYPT    "$y$"
//#define FANCY_CRYPTHELPER_PREFIX_SCRYPT      "$7$"
//#define FANCY_CRYPTHELPER_PREFIX_BCRYPT      "$2b$"

// 2x supported:
#define FANCY_CRYPTHELPER_PREFIX_SHA256CRYPT "$5$"
#define FANCY_CRYPTHELPER_PREFIX_SHA512CRYPT "$6$"


/**
 * Crypts ("hashes") the specified password using crypt_rn() (and crypt_gensalt_rn() for the salt).
 *
 * Throws up if problems occur, like unsupported prefix, password too short or long (4..256), ...
 *
 * The results shoud be compatible with openssl passwd, i.e.:
 *   cryptPassword("test123", FANCY_CRYPTHELPER_PREFIX_SHA256CRYPT)
 * is 'compatible' to
 *   openssl passwd -5 test123
 * and
 *   cryptPassword("test123", FANCY_CRYPTHELPER_PREFIX_SHA512CRYPT)
 * is 'compatible' to
 *   openssl passwd -6 test123
 *
 *
 * newPassword is the (new) password to crypt (hash), length must be in range 4..256.
 * prefix      is an optional prefix for the crypt meanchism to use, see FANCY_CRYPTHELPER_PREFIX_* above.
 *             If specified empty FANCY_CRYPTHELPER_PREFIX_SHA512CRYPT resp. "$6$" is used.
 * count       is the count (rounds) parameter going into crypt_gensalt_rn().
 *             The default for sha256crypt and sha512crypt is 5000 but said to be too low, see man 5 crypt.
 */
std::string FANCYBASE_API cryptPassword(
        const char* newPassword,
        const std::string & prefix="",
        int count=5000);

/**
 * Uses crypt_rn to verify the specified password against the specified crypted ("hashed") password.
 */
bool FANCYBASE_API verifyPassword(
        const char* passwordToVerify,
        const std::string & crytedPassword);


} // namespace CryptHelper

} // namespace Fancy


#endif // not WIN32

#endif /* FANCY_CYRYPT_HELPER_H_ */
