/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDIRECTORY_H_
#define FANCYDIRECTORY_H_

#include <fancyobject.h>

#ifdef WIN32
#define FANCY_DIR_SEPARATOR_CHAR '\\'
#define FANCY_DIR_SEPARATOR_STR "\\"
#define FANCY_PATH_SEPARATOR_CHAR ';'
#define FANCY_PATH_SEPARATOR_STR ";"
#else
#define FANCY_DIR_SEPARATOR_CHAR '/'
#define FANCY_DIR_SEPARATOR_STR "/"
#define FANCY_PATH_SEPARATOR_CHAR ':'
#define FANCY_PATH_SEPARATOR_STR ":"
#endif

namespace Fancy {

class Iterator;
class Directory;

/**
 * A directory entry describing a regular file, a directory or
 * some kind of special file.
 *
 * This class represents an abstraction of the fstat(2) unctionality
 * under UNIX.
 */
class FANCYBASE_API DirEntry: public virtual Fancy::Object {
protected:
    DirEntry();
    virtual ~DirEntry();
public:
    typedef ObjPtr<DirEntry> ptr_t;

    /**
     * Return a new instance of the given absolute or relative path.
     *
     * This function does not throw an exception, if the given path does not
     * exist.
     *
     * Instead, it returns an instance, which
     * returns @c false in @c exists().
     */
    static ptr_t newInstance(const std::string &path);

    /**
     * @return Whether this entry exists in the file system.
     */
    virtual bool exists() const = 0;

    /**
     * Remove the given entry from the file system. If this entry is a directory,
     * the directory has to be empty.
     * To erase a non-empty directory, use Directory::eraseRecursive()
     */
    virtual void erase() = 0;

    /**
     * @return Whether this entry exists and may be read by the current user.
     * By using stat over lstat in Unix this info refers to the end of an eventual symlink chain.
     */
    virtual bool isReadable() const = 0;

    /**
     * @return Whether this entry exists and may be written by the current user.
     * By using stat over lstat in Unix this info refers to the end of an eventual symlink chain.
     */
    virtual bool isWriteable() const = 0;

    /**
     * @return Whether this entry exists and is an ordinary file.
     * By using stat over lstat in Unix this info refers to the end of an eventual symlink chain.
     */
    virtual bool isFile() const = 0;

    /**
     * @return Whether this entry exists and is a directory. If true, this entry additinally
     *         implements the @c Fancy::Directory interface.
     * By using stat over lstat in Unix this info refers to the end of an eventual symlink chain.
     */
    virtual bool isDirectory() const = 0;

    /**
     * @return Whether this entry exists and is a special file like a pipe, unix socket
     *         or a device entry.
     * By using stat over lstat in Unix this info refers to the end of an eventual symlink chain.
     */
    virtual bool isSpecial() const = 0;

#ifndef WIN32
    /**
     * @return Whether this entry exists and is a symbolic link.
     * By using lstat over stat in Unix this info refers to the original file.
     *
     * ATTENTION: The Windows implementation always returns false.
     * Function might be removed for Windows with the next major upgrade
     */
    virtual bool isSymlink() const = 0;

    /**
     * @return The UNIX file permissions. In fact, the @c st_mode
     *         member of @c strut stat is returned, which bears additional
     *         bits for querying the type of this directory entry.
     *
     * By using stat over lstat in Unix this info refers to the end of an eventual symlink chain.
     * Use getOriginMode() to use the lstat() data.
     */
    virtual int getMode() const = 0;

    /**
     * Change the UNIX file permissions.
     */
    virtual void chmod(int mode) = 0;
#endif

    /**
     * If the entry exists, and is a symbolic link, follows the symbolic link once and returns that new instance.
     * Otherwise returns this instance.
     *
     * ATTENTION: There is no protection against circular symlink chains, so when looping you MUST use some endless-loop-protection.
     *
     * ATTENTION: No actual following is implemented for Windows.
     */
    virtual ptr_t followLink() const = 0;

    /**
     * @return The size of this entry, if it is an ordinary file.
     */
    virtual uint64_t getSize() const = 0;

    /**
     * @return The last access time in milliseconds since the epoch.
     */
    virtual uint64_t getLastAccessMillis() const = 0;

    /**
     * @return The last modification time in milliseconds since the epoch.
     */
    virtual uint64_t getLastModificationMillis() const = 0;

    /**
     * @return The creation time in milliseconds since the epoch.
     */
    virtual uint64_t getCreationMillis() const = 0;

    /**
     * @return The name of this entry, which does not contain any
     *         directory information.
     */
    virtual std::string getName() const = 0;

    /**
     * @return The extension (~ file type hint) of this entry.
     *         Empty string if getName() does not contain a dot ('.') or ends with one.
     */
    virtual std::string getExtension() const = 0;

    /**
     * @return The (possibly relative) path of this entry.
     */
    virtual std::string getPath() const = 0;

    /**
     * @return The absolute path of this entry.
     */
    virtual std::string getAbsolutePath() const = 0;

    /**
     * @return The directory, which contains this entry.
     */
    virtual ObjPtr<Directory> getParent() const = 0;


// TODO FancyLibs4: move all following methods up where they belong

#ifndef WIN32

    /**
     * @return Whether this entry is executable.
     * By using stat over lstat in Unix this info refers to the end of an eventual symlink chain.
     */
    virtual bool isExecutable() const = 0;

    /**
     * @return The UNIX file permissions. In fact, the @c st_mode
     *         member of @c strut stat is returned, which bears additional
     *         bits for querying the type of this directory entry.
     *
     * By using lstat over stat in Unix this info refers to original entry.
     * Use getMode() to use the stat() data which refer to the end of an eventual symlink chain.
     */
    virtual int getOriginMode() const = 0;

#endif

    // WISHLIST: getTempDir(), getTempPath() or the like

};


/**
 * An abstraction of a directory in the file system.
 */
class FANCYBASE_API Directory: public virtual DirEntry {
protected:
    Directory();
    virtual ~Directory();
public:

    typedef ObjPtr<Directory> ptr_t;

    /**
     * Create a new instance. This method does not throw an exception,
     * if the directory does not exist. Instead, it returns an instance, which
     * returns @c false in @c exists(). This allows the user to
     * create a missing directory using @c mkdir(), if a directory does
     * not exist.
     */
    static ptr_t newInstance(const std::string &path);

    /**
     * @return An instance, which represents the current working directory.
     */
    static ptr_t getCwd();

    /**
     * Create a new instance with a path relative to this directory.
     * This method does not throw an exception,
     * if the directory does not exist. It returns an instance, which
     * return @c false in @c exists(). This allows the user to
     * create a missing directory using @c mkdir(), if a directory does
     * not exist.
     */
    virtual ptr_t newSubDirInstance(const std::string &path) const = 0;

    /**
     * Make this directory the current working directory of the current
     * process.
     */
    virtual void chdir() = 0;

    /**
     * Create this directory with default permissions. The default permissions
     * are operating system dependent. Under UNIX the permissions are set to 0750.
     * Intermediate directories are not created automatically, use mkdirs() for that.
     */
    virtual void mkdir() = 0;

    /**
     * Create this directory and all intermediate parent directories with default
     * permissions. The default permissions are operating system dependent.
     * Under UNIX the permissions are set to 0750.
     */
    virtual void mkdirs() = 0;

#ifndef WIN32
    /**
     * Create this directory with the given UNIX permissions.
     * Intermediate directories are not created automatically, use mkdirs(int mode) for that.
     */
    virtual void mkdir(int mode) = 0;

    /**
     * Create this directory and all intermediate parent directories with
     * the given UNIX the permissions.
     */
    virtual void mkdirs(int mode) = 0;
#endif

    /**
     * Erase this directory by previously erasing all entries in this
     * directory. Subdirectories a are erased recursively.
     */
    virtual void eraseRecursive() = 0;

    /**
     * @return A directory entry with a path name relative to this directory.
     *         This function does not check, if an entry with the given name exists.
     */
    virtual DirEntry::ptr_t getEntry(const std::string &name) const = 0;

    /**
     * @return An iterator, which allows to iterate through all entries
     *         in this directory. The key of the returned iterator is a string
     *         representing the name of the entry, the value is a
     *         @c Fancy::DirEntry with all relevant data about the file or
     *         subdirectory.
     */
    virtual ObjPtr<Iterator> findAllEntries() const = 0;


    /**
     * @return An iterator, which allows to iterate through all entries
     *         in this directory, which match the given pattern. The pattern
     *         may contain the usual wildcard characters like '*'.
     *         The key of the returned iterator is a Fancy::String
     *         representing the name of the entry, the value is a
     *         @c Fancy::DirEntry with all relevant data about the file or
     *         subdirectory.
     */
    virtual ObjPtr<Iterator> findEntries(const std::string &pattern) const = 0;
};

}

#endif /* FANCYDIRECTORY_H_ */
