/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYENUM_H_
#define FANCYENUM_H_

#include <fancyserializable.h>

namespace Fancy {

/**
 * A virtual base class for JAVA enum objects.
 *
 * Implementations of this class should provide the following members:
 * @verbatim
 *   enum enum_t {
 *     ENUM_VALUE0,
 *     ENUM_VALUE1,
 *     ....
 *   };
 *
 *   const char* getSerialID() const;
 *   Serializable::ptr_t resolveRead(int32_t v) const;
 *   int32_t getMaxValue() const;
 *   static ptr_t newInstance(enum_t e);
 *   static ptr_t newInstance(const std::string &s);
 *   static Serializable::ptr_t newSerializableInstance();
 *   static const char *getSerialID_s();
 *   static const char *getSuperSerialID_s();
 * @endverbatim
 */
class FANCYBASE_API Enum: public Serializable {

    int32_t v;

protected:
    Enum(int32_t _v) : v(_v) {}

    ~Enum();

    /**
     * Resolve a static enum instance.
     */
    virtual Serializable::ptr_t resolveRead(int32_t v) const = 0;

public:
    typedef ObjPtr<Enum> ptr_t;

    static const char *getSerialID_s();

    /**
     * Return the C++ enum value.
     */
    int32_t getValue() const { return this->v; }

    /**
     * The C++ enum value with the highest ordinary integral value.
     */
    virtual int32_t getMaxValue() const = 0;

    std::string toString() const;

    const char* getSerialID() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    // implementation of Fancy::Serializable
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    Serializable::ptr_t readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);
};
}

#endif /* FANCYENUM_H_ */
