/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A thread-safe reference counting pointer to objects not derived from
** Fancy::Object.
**
*****************************************************************************/

#ifndef FANCYBASE_PTR_H
#define FANCYBASE_PTR_H

#include <fancyobject.h>

namespace Fancy {

  template <class T> class Ptr
  {
    T *t;
    Object *ref;

    void _release_ref() {
      if (ref && ref->deref()) delete t;
    }

  public:
    Ptr() : t(0),ref(0) {}
    Ptr(T *_t) : t(_t),ref(_t ? new Object() : 0) { if (ref) ref->ref(); }

    Ptr(const Ptr &a) : t(a.t),ref(a.ref) { if (ref) ref->ref(); }

    template<class U>Ptr(const Ptr<U> &a) : t(dynamic_cast<T*>(a.get()))
    { if (t) { ref = a.getRef(); ref->ref(); } else ref = 0; }

    Ptr &operator=(const Ptr &a) {
      _release_ref();
      t=a.t;
      ref=a.ref;
      if (ref) ref->ref();
      return *this;
    }

    template<class U> Ptr &operator=(const Ptr<U> &a) {
      _release_ref();
      t=dynamic_cast<T*>(a.get());
      if (t) {
        ref=a.getRef();
        ref->ref();
      }
      else
        ref = 0;
      return *this;
    }

    Ptr &operator=(T *_t) {
      _release_ref();
      t = _t;
      ref=_t ? new Object() : 0;
      if (ref) ref->ref();
      return *this;
    }

    void setNull() {
      _release_ref();
      t = 0;
      ref = 0;
    }

    ~Ptr() { _release_ref(); }

    bool isNull() const { return t == 0; }
    T *operator->() const { return t; }
    T &operator*() const { return *t; }

    /*!
       Indirection of pointer. Take care of not calling Ptr(t) with the
       returned pointer. This method should normally not be used, because
       the lifetime of returned pointer is tied to the lifetime of this
       Ptr object.
     */
    T *get() const { return t; }
    /*!
       Indirection of reference counter. This method should normally not be used, because
       the reference is managed by the liecycle methods of this class.
     */
    Object *getRef() const { return ref; }

    bool operator!() const {
      return t == 0;
    }

    template<class U> bool operator== (const Ptr<U> &a) {
      return a.get() == this->get();
    }

    template<class U> bool operator!= (const Ptr<U> &a) {
      return a.get() != this->get();
    }

    template<class U> bool operator< (const Ptr<U> &a) {
      return a.get() < this->get();
    }
  };

  template<class T>
  inline std::ostream& operator<<(std::ostream& os, const Ptr<T> &p) {

    if (p.isNull()) return writeObjectPtr(os,0);
    else return os << (*p);
  }

}

#endif
