/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSERIALIZABLEFACTORY_H_
#define FANCYSERIALIZABLEFACTORY_H_

#include <fancyserializable.h>
#include <fancylist.h>

namespace Fancy {

/*!
 * The interface for a factory of serializable objects.
 */
class FANCYBASE_API SerializableFactory: public virtual Fancy::Object {
public:
    typedef Fancy::ObjPtr<SerializableFactory> ptr_t;

    /**
     * Return a new default implementation. The returned instance is aware of all
     * Serializable classes in the library @c fancybase.
     */
    static ptr_t newInstance();

    /**
     * Return a new default implementation using the given delegate for resolving unknown
     * serial IDs.
     *
     * The returned instance is not aware of any Serializable class in the library @c fancybase.
     */
    static ptr_t newInstance(const ptr_t &delegate);

    /*!
     *
     * @param id The object ID as returned by \c Serializable::getSerialID().
     * @param serialVersionUID The object version ID as returned by
     *                         \c Serializable::getSerialVersionUID().
     * @return a new object of the given type id.
     */
    virtual ObjPtr<Serializable> newSerializable(const std::string &id, int64_t serialVersionUID) const = 0;

    /**
     * @param id The object ID as returned by @c Serializable::getSerialID().
     * @param serialVersionUID The object version ID as returned by
     *                         @c Serializable::getSerialVersionUID().
     * @param sz The size of the returned array.
     * @return an instance of @c Fancy::ArrayObjectList<T> where T is the of the given type id.
     */
    virtual ObjPtr<List> newSerializableArray(const std::string &id, int64_t serialVersionUID, size_t sz)  const = 0;

    /*!
     * @param id The object ID as returned by \c Serializable::getSerialID().
     * @return a new object of the given type id.
     */
    virtual ObjPtr<Serializable> newSerializable(const std::string &id) const = 0;

    /**
     * @param id The object ID as returned by @c Serializable::getSerialID().
     * @param sz The size of the returned array.
     * @return an instance of @c Fancy::ArrayObjectList<T> where T is the of the given type id.
     */
    virtual ObjPtr<List> newSerializableArray(const std::string &id, size_t sz)  const = 0;

    /*!
     * @param id The object ID as returned by \c Serializable::getSerialID().
     * @return Whether the given type id is registered in this factory.
     */
    virtual bool isSerializableRegistered(const std::string &id) const = 0;

    /*!
     * @param id The object ID as returned by \c Serializable::getSerialID().
     * @return Whether the given type id is registered as array type in this factory.
     */
    virtual bool isSerializableArrayRegistered(const std::string &id) const = 0;

    /*!
     * @param id The object ID as returned by \c Serializable::getSerialID().
     * @return The default serialVersionUID of the given class.
     */
    virtual int64_t getDefaultSerialVersionUID(const std::string &id) const = 0;

    /*!
     * Return the super serial ID of the given class or an empty string, if
     * the given type ID has no super class except Fancy::Object.
     */
    virtual std::string getSuperSerialID(const std::string &id) const = 0;

    /*!
     * Return whether the given object or one of it's super classes is of the
     * given type.
     *
     * If the given object pointer in null, false is returned.
     */
    virtual bool instanceof(const char *id, const ObjPtr<Object> &o) const = 0;

    /*!
     * Return whether objects of the the given serial ID or one of it's super classes
     * is of the given type.
     *
     * If the given object pointer in null, false is returned.
     */
    virtual bool instanceof(const char *targetSerialId, const char *sourceSerialId) const = 0;


    /**
     * Register a serializable class using a constructor function.
     */
    virtual void registerClass(const char *id, int64_t serialVersionUID, newSerializableInstance_t ctor,
    		const char* superId = 0) = 0;

    /**
     * Register a serializable class using a constructor function and a set of compatibility serial version UIDs.
     */
    virtual void registerClass(const char *id, int64_t serialVersionUID,
                               getOldSerialVersionUIDs_s_t getOSVUIDs,
                               newSerializableInstance_t ctor,
                               const char* superId = 0) = 0;

    /**
     * Register an abstract class for inspecting inheritance relationships.
     */
    virtual void registerAbstractClass(const char *id, int64_t serialVersionUID, const char* superId) = 0;

    /**
     * Register a serializable class array using a constructor function.
     */
    virtual void registerClassArray(const char *id, int64_t serialVersionUID,
                                    newArrayObjectListInstance_t newArrayObjectList) = 0;

    /**
     * Register a serializable class array using a constructor function and a set of compatibility serial version UIDs.
     */
    virtual void registerClassArray(const char *id, int64_t serialVersionUID,
                                    getOldSerialVersionUIDs_s_t getOSVUIDs,
                                    newArrayObjectListInstance_t newArrayObjectList) = 0;

    /**
     * Register a serializable class by its static @c getSerialID_s, @c getSerialVersionUID_s
     * and @c newSerializableInstance functions.
     */
    template <class T> void registerSerializable() {
        this->registerClass(T::getSerialID_s(),
        		T::getSerialVersionUID_s(),T::getOldSerialVersionUIDs_s,
        		T::newSerializableInstance,
        		T::getSuperSerialID_s());
    }

    /**
      * Register a serializable class by its static @c getSerialID_s, @c getSerialVersionUID_s
      * and @c newSerializableInstance functions.
      */
     template <class T> void registerAbstractSerializable() {
         this->registerAbstractClass(T::getSerialID_s(),
         		T::getSerialVersionUID_s(),
         		T::getSuperSerialID_s());
     }

     /**
     * Register a serializable class as array component by its static @c getSerialID_s and
     * @c getSerialVersionUID_s functions as well as @c ObjectArray<T>::newListInstance.
     */
    template <class T> void registerSerializableArray() {
        this->registerClassArray(T::getSerialID_s(),T::getSerialVersionUID_s(),
                                 T::getOldSerialVersionUIDs_s,ObjectArray<T>::newListInstance);
    }
    /**
     * Register a serializable class for normal fields and array components by calling
     * @c registerSerializable() as well as @c registerSerializableArray().
     */
    template <class T> void registerSerializableAll() {
        this->registerSerializable<T>();
        this->registerSerializableArray<T>();
    }

protected:
    SerializableFactory();
    ~SerializableFactory();
};

}

#endif /* FANCYSERIALIZABLE_H_ */
