/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYUTCTIMESTAMP_H_
#define FANCYUTCTIMESTAMP_H_

#include <fancynumbers.h>
#include <time.h>

namespace Fancy {

class Integer;

class FANCYBASE_API UtcTimestamp: public virtual Fancy::Serializable {

    uint64_t utcMillis;
    Integer::ptr_t tz_off;

public:
    typedef ObjPtr<UtcTimestamp> ptr_t;

    static Serializable::ptr_t newSerializableInstance();

    /** creates a new instance with the current system time (in milliseconds from epche) and time zone offset information */
    static ptr_t newInstance();

    /** Creates a new instance with the date+time specified in the iso8601 string.
     * If no timezone part is found, UTC is assumed and getTzOffset().isNull() (rather than 0!).
     * We are thinking about changing this behaviour because ISO8601 and XML-Schema rather presume local time.
     */
    static ptr_t newInstance(const char *iso8601);

    /** Creates a new instance with the date+time specified in the iso8601 string.
     * If no timezone part is found, UTC is assumed and getTzOffset().isNull() (rather than 0!).
     * We are thinking about changing this behaviour because ISO8601 and XML-Schema rather presume local time.
     */
    static ptr_t newInstance(const std::string &iso8601);

     /** For semantics of fmt, see documentation of std::string format(const char *fmt) const.
      * A reminder though: If a literal character is expected, like 'T' in some ISO8601 "dialect", escape it with a '\\'!
      */
    static ptr_t newInstance(const char *str, const char *fmt);

    static ptr_t newInstance(int tz_off, const struct tm *tm, int millis);

    /** creates a new instance with the given broken down time with no
     * timezone information.
     */
    static ptr_t newInstance(const struct tm *tm, int millis = 0);

    /** creates a new instance with the given timestamp time (in milliseconds since the epoch)
     * with the given possibly null timezone offset in minutes.
     *
     * For real local time, use this:
     *   int tz_offset;
     *   uint64_t nowMillis = Fancy::getCurrentTimeMillisTZ(&tz_offset);
     *   Fancy::UtcTimestamp::ptr timestamp = Fancy::UtcTimestamp.newInstance(millis, tz_offset);
     */
    static ptr_t newInstance(uint64_t millis, const Integer::ptr_t &tz_off);

    /** creates a new instance with the given timestamp time (in milliseconds since the epoch)
     * with the given timezone offset in minutes.
     *
     * For real local time, use this:
     *   int tz_offset;
     *   uint64_t nowMillis = Fancy::getCurrentTimeMillisTZ(&tz_offset);
     *   Fancy::UtcTimestamp::ptr timestamp = Fancy::UtcTimestamp.newInstance(millis, tz_offset);
     */
    static ptr_t newInstance(uint64_t millis, int tz_off);

    /** creates a new instance with the given timestamp time (in milliseconds since the epoch) without(!) timezone information.
     * Leaving out timezone informations suppresses any output for TZD or TZd when using format().
     * For "real" GMT time (with tz offset 0) use newInstance(millis, (int)0);
     * For real local time, use this:
     *   int tz_offset;
     *   uint64_t nowMillis = Fancy::getCurrentTimeMillisTZ(&tz_offset);
     *   Fancy::UtcTimestamp::ptr timestamp = Fancy::UtcTimestamp.newInstance(millis, tz_offset);
     */
    static ptr_t newInstance(uint64_t millis);

    /** creates a new instance with the given timestamp time (in milliseconds since the epoch) with a
     * timezone information in the system's timezone.
     */
    static ptr_t newLocalInstance(uint64_t millis);

    /** Creates a new instance from a timestamp given in the format specified in
     * <a href="http://tools.ietf.org/html/rfc822#section-5">Section 5 of RFC822</a>.
     *
     * The timezone information is adapted to the timezone designator given in the
     * passed string.
     *
     * Such timestamps frequently appear in HTTP headers, so this justifies
     * the existence of a separate method for parsing those.
     */
    static ptr_t fromRFC822(const char *str);

    /** Creates a new instance from a timestamp given in the format specified in
     * <a href="http://tools.ietf.org/html/rfc822#section-5">Section 5 of RFC822</a>.
     *
     * The timezone information is adapted to the timezone designator given in the
     * passed string.
     *
     * Such timestamps frequently appear in HTTP headers, so this justifies
     * the existence of a separate method for parsing those.
     */
    static ptr_t fromRFC822(const std::string &str);

    static const int ISO8601_DATE_FORMAT = 0;            // YYYY-MM-DD
    static const int ISO8601_DATETIME_FORMAT = 1;        // YYYY-MM-DDThh:mm:ss, +TZD if tz_offset != 0
    static const int ISO8601_DATETIMEMILLIS_FORMAT = 2;  // YYYY-MM-DDThh:mm:ss.zzz, +TZD if tz_offset != 0

    /**
     * @return The ISO8601 string either in the format
     *         @c ISO8601_DATE_FORMAT, @c ISO8601_DATETIME_FORMAT or
     *         @c ISO8601_DATETIME_MILLIS_FORMAT.
     */
    std::string toISO8601(int fmt) const;

    /**
     * @return The ISO8601 string in the format @c ISO8601_DATETIME_MILLIS_FORMAT.
     */
    std::string toString() const;

    /**
     * @return a timestamp formatted according to
     * <a href="http://tools.ietf.org/html/rfc822#section-5">Section 5 of RFC822</a>.
     */
    std::string toRFC822() const;

    /**
     * Formats this timestamp ac. to the given format.
     * The fmt interpretation is unlike printf(2) but rather like QDateTime.toString() or Java DateFormats:
     * YYYY  is replaced by the full year with century with 4 digits, i.e. 1970, 2010, ...
     * YY    is replaced by the year-of-century with 2 digits, 00-99
     * Y     is replaced by the year straight, 1-4 digits
     * MM    is replaced by the month of year with 2 digits, 01-12
     * M     is replaced by the month of year straight, 1-2 digits, 1-12
     * DD    is replaced by the day of month with 2 digits, 01-31
     * D     is replaced by the day of month straight, 1-2 digits, 1-31
     * WW    is replaced by the day of week with 2 digits, 00-06
     * W     is replaced by the day of month straight, 1 digit, 0-6
     * hh    is replaced by the hour with 2 digits, 00-23
     * h     is replaced by the hour straight, 1-2 digits, 0-23
     * mm    is replaced by the minute with 2 digits, 00-59
     * m     is replaced by the minute straight, 1-2 digits, 0-59
     * ss    is replaced by the seconds (of minute) with 2 digits, 00-59
     * s     is replaced by the seconds (of minute) straight, 1-2 digits, 0-59
     * zzz   is replaced by the millisecond (of second) with 3 digits, 000-999
     * TZD   if this was created with timezone infos, is replaced by Z or offset in +hh:mm or -hh:mm format, i.e. +01:00.
     * TZd   if this was created with timezone infos, is replaced by offset in +hhmm or -hhmm format (i.e. +0100 and +0000 instead of Z).
     * Tzd   if this was created with timezone infos, is replaced by offset in +hh or -hh format (i.e. +01 and +00 instead of Z), conforming to ISO 8601/SQL standard.
     * \\    escapes the following char, i.e. \\Y get's passed as 'Y' rather than beeing replaced by the year
     *
     * @param fmt the format string
     * @return the formated timestamp
     */
    std::string format(const char *fmt) const;

    bool equals(const Object &o) const;
    bool lessThan(const Object &o) const;

    uint64_t       getUtcMillis() const { return this->utcMillis; }
    Integer::ptr_t getTZOff()  const { return this->tz_off; }

    int getMilliSecond() const;
    int getSecond() const;
    int getMinute() const;
    int getHour() const;
    int getDay() const;
    int getMonth() const;
    int getYear() const;
    int getDayOfYear() const;
    int getDayOfWeek() const;

    void  to_c_tm(struct tm *tm) const;

    // implementation of Fancy::Serializable
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

    /**
     * @param y The year in the range from 1759 to 9999.
     * @return Whether the given year is a leap year in the gregorian calendar.
     */
    static bool isLeapYear(int y);

protected:
    UtcTimestamp(uint64_t millis, const Integer::ptr_t &tz_off);
    UtcTimestamp(uint64_t millis);
    virtual ~UtcTimestamp();
};

}

#endif /* FANCYUTCTIMESTAMP_H_ */
