/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Christoph Lechleitner/IBCL <lech@ibcl.at>
 *
 ************************************************************/

#ifndef FANCY_ZIPCHANNEL_H_
#define FANCY_ZIPCHANNEL_H_

#include <fancychannel.h>
#include <fancyzipentry.h>

namespace Fancy {

/**
 * A Channel that creates ZIP64 files.
 *
 * This implementation is NOT thread-safe.
 * The data is compressed using "deflate" compression.
 *
 * Please note, that you have to call close() or flush() to really finish the
 * generated output stream.
 */
class FANCYBASE_API ZipChannel : public virtual Channel {

public:
    typedef ObjPtr<ZipChannel> ptr_t;

    /**
     * Returns a ZipChannel instance.
     *
     * channel ... The channel, where to store the zip file's data
     * comment ... ZIP file comment with at most 65.535 Bytes
     */
    static ptr_t newInstance(const Fancy::Channel::ptr_t &channel,
    	const std::string &comment = "");

    /**
	 * Not supported! Throws a Fancy::Exception if used
	 */
	virtual int read(void *buf, int count) = 0;

	/**
	 * Writes the data of a ZipEntry to the channel.
	 *
	 * If no Fancy::ZipEntry has been set using
	 * Fancy::ZipChannel::putNextEntry() or the channel has been flushed without
	 * setting a new Fancy::ZipEntry, an Fancy::Exception is thrown.
	 */
	virtual int write(const void *buf, int count) = 0;

    /**
     * Sets the next Fancy::ZipEntry
     *
     * This function must be called prior to writing to the channel using
     * Fancy::ZipChannel::write().
     * This method automatically flushes the channel using
     * Fancy::ZipChannel::flush() before writing the new ZipEntry to the
     * channel.
     */
    virtual void putNextEntry(const Fancy::ZipEntry::ptr_t &entry) = 0;

    /**
     * Flushes all internal buffers and finalizes the current Fancy::ZipEntry.
     *
     * Subsequent calls of Fancy::ZipChannel::write() will throw a
     * Fancy::Exception, as long as no new Fancy::ZipEntry is added to the
     * Fancy::ZipChannel using Fancy::ZipChannel::putNextEntry().
     */
    virtual void flush() = 0;

    /**
     * Flushes all internal buffers (see Fancy::ZipChannel::flush()) and
     * finalizes the ZIP file by writing the central directory structure to the
     * internal channel.
     * Writing to the channel after it was closed results in undefined behavior
     */
    virtual void close() = 0;

protected:
    ZipChannel();
    virtual ~ZipChannel();
};

}

#endif /* FANCY_DEFLATECHANNEL_H_ */
